# Generated by Django 5.1.7 on 2025-03-17 11:04

from django.db import migrations

import djangocms_link.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0018_remove_link_anchor_remove_link_external_link_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='link',
            field=djangocms_link.fields.LinkField(default=dict, help_text='-', verbose_name='Link'),
        ),
    ]
