# Generated by Django 4.2.16 on 2024-10-24 19:59

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("djangocms_link", "0017_link_link"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="link",
            name="anchor",
        ),
        migrations.RemoveField(
            model_name="link",
            name="external_link",
        ),
        migrations.RemoveField(
            model_name="link",
            name="file_link",
        ),
        migrations.RemoveField(
            model_name="link",
            name="internal_link",
        ),
        migrations.RemoveField(
            model_name="link",
            name="mailto",
        ),
        migrations.RemoveField(
            model_name="link",
            name="phone",
        ),
    ]
